package com.cicdi.notify.wechat.corp;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.cicdi.notify.DefaultNotifyType;
import com.cicdi.notify.Notifier;
import com.cicdi.notify.NotifyType;
import com.cicdi.notify.Provider;
import com.cicdi.notify.wechat.WechatProvider;
import org.apache.http.NameValuePair;
import org.apache.http.client.methods.CloseableHttpResponse;
import org.apache.http.client.methods.HttpGet;
import org.apache.http.client.methods.HttpPost;
import org.apache.http.client.utils.URIBuilder;
import org.apache.http.entity.StringEntity;
import org.apache.http.impl.client.CloseableHttpClient;
import org.apache.http.impl.client.HttpClientBuilder;
import org.apache.http.message.BasicNameValuePair;
import org.apache.http.util.EntityUtils;

import java.io.IOException;
import java.time.Duration;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;

public class WechatCorpNotifier implements Notifier<WechatCorpTemplate> {

    private String accessToken;

    private long refreshTokenTime;

    private final long tokenTimeOut = Duration.ofSeconds(7000).toMillis();

    private static final String tokenApi = "https://qyapi.weixin.qq.com/cgi-bin/gettoken";

    private static final String notifyApi = "https://qyapi.weixin.qq.com/cgi-bin/message/send";

    private final WechatCorpProperties properties;

    private final String notifierId;

    public WechatCorpNotifier(String id, WechatCorpProperties properties) {
        this.properties = properties;
        this.notifierId = id;
    }

    @Override
    public String getNotifierId() {
        return notifierId;
    }

    @Override
    public NotifyType getType() {
        return DefaultNotifyType.wechat;
    }

    @Override
    public Provider getProvider() {
        return WechatProvider.corpMessage;
    }

    @Override
    public void send(WechatCorpTemplate template, Map<String, Object> context) {
        CloseableHttpClient httpClient = HttpClientBuilder.create().build();
        HttpPost httpPost = new HttpPost(notifyApi + "?access_token=" + getToken());
        httpPost.setHeader("Content-Type", "application/json;charset=utf8");

        StringEntity stringEntity = new StringEntity(template.createJsonRequest(context), "UTF-8");
        httpPost.setEntity(stringEntity);

        try {
            CloseableHttpResponse response = httpClient.execute(httpPost);
            String result = EntityUtils.toString(response.getEntity());
            System.out.println(result);
            JSONObject jsonObject = JSON.parseObject(result);
            if (!"0".equals(jsonObject.get("errcode").toString())) {
                throw new RuntimeException("发送微信企业通知失败：" + jsonObject.get("errmsg"));
            }
        } catch (IOException e) {
            e.printStackTrace();
        }
    }

    private String getToken() {
        if (System.currentTimeMillis() - refreshTokenTime > tokenTimeOut || accessToken == null) {
            return requestToken();
        }
        return accessToken;
    }

    private String requestToken() {
        List<NameValuePair> params = new ArrayList<>();
        params.add(new BasicNameValuePair("corpId", properties.getCorpId()));
        params.add(new BasicNameValuePair("corpSecret", properties.getCorpSecret()));

        CloseableHttpClient httpClient = HttpClientBuilder.create().build();
        try {
            HttpGet httpGet = new HttpGet(new URIBuilder(tokenApi).setParameters(params).build());
            CloseableHttpResponse response = httpClient.execute(httpGet);
            JSONObject responseJson = JSON.parseObject(EntityUtils.toString(response.getEntity()));
            if (responseJson.containsKey("access_token")) {
                String access_token = responseJson.get("access_token").toString();
                accessToken = access_token;
                refreshTokenTime = System.currentTimeMillis();
                return access_token;
            } else {
                throw new RuntimeException("获取令牌失败：" + responseJson.getString("errmsg"));
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
        return null;
    }

    @Override
    public void close() {
        this.accessToken = null;
        this.refreshTokenTime = 0;
    }
}
